/* BEGIN COPYRIGHT */
/* END COPYRIGHT */
/* BEGIN GOVERNMENT_RIGHTS */
/***********************************************************************
 * UNLIMITED RIGHTS
 * This material may be reproduced by or on behalf of the
 * U.S. Government pursuant to the copyright license under the DFAR
 * 252.227-7014 which was made part of the above contract.
 ***********************************************************************/
/* END GOVERNMENT_RIGHTS */

/* BEGIN ITAR */
/* END ITAR */

/* BEGIN CATEGORY */
/************************************************************************** 
 * CATEGORY: 1 (see SEN for definitions)
 *
 * (Calculate_VegIndex.cpp)
 *
 * Refer to Software Standards and Practices Manual (SSPM) for 
 * coding standards.
 *
 **************************************************************************/
/* END CATEGORY */

/**************************************************************************
 * 
 * NAME: Calculate_VegIndex
 *
 * DESCRIPTION: Function for computing the vegetation indices that
 *              compose the NPOESS/VIIRS Vegetation Index EDR.
 *
 *
 * USAGE: See header file for more information.
 *
 * Parameter         Type          I/O    Description
 * -----------       -----         ---    ----------------
 * See the method descriptions located in the .h file for details.
 *
 *
 * REFERENCES:
 * Y2400  - Vegetation Index Algorithm Theoretical Basis Document
 * Y2474  - Land Module Level Software Architecture Document
 * Y2499  - VIIRS Vegetation Index Unit Level Detailed Design Document
 * Y10883 - VIIRS Land Module Data Dictionary
 * D36951 - Operational Algorithm Document for the VIIRS Vegetation Index EDR
 *          Software
 *
 * LIMITATIONS: 
 * none
 *
 *
 * HISTORY:
 *   DATE:       PR#         AUTHOR       Description
 *   --------    ------      --------     -----------------
 *   02FEB2002               smiller      Initial Version
 *   18SEP2003               mhoover      Ported to IPO model
 *   10MAR2004   1500        mhoover      Added check for missing data
 *   31MAR2004   1778        mhoover      Moved lqf check here, fixed 
 *                                        error handling and 
 *                                        initialization
 *   28APR2004   2051        mhoover      Modified fill value, moved 
 *                                        lqf check outside loop
 *   29JUN2004               mschmidt     Added code to set EDR pixels
 *                                        to fill values when the 
 *                                        pixel is not processed.
 *   16FEB2005               C. Erkes     Updated prolog
 *   11MAY2005               C. Erkes     Updated fill values and added DQN
 *   26MAY2005               B. Nye       Updated ITAR markings
 *   09JUN2005   7487        B. Nye       Added test for M3 > fill value 
 *   08FEB2006               J. Alers     Updated debug interfaces
 *   28MAR2006   9351        D.Hrivnak    Updated debug log levels
 *   25JUL2006               C. Erkes     Addd quality flags to match EDRPR   
 *                                        v1.8
 *   16AUG2006               C. Erkes     Updated EVI range to be [-1;4] as 
 *                                        requested by Tech Memo 
 *                                        NP-EMD-2006.510.0042
 *   18OCT2006               C. Erkes     Updated scaling of NDVI and EVI 
 *   06MAY2007               Henderson    Updated to use ProCmnProductDictionary
 *                                        for scaling
 *   15MAY2007               Henderson    Updated to initalize granule level
 *                                        flag
 *   23MAY2007               Hoover       Modified loop indexes for image data
 *   10DEC2007   16155       T. Gardner   Corrected extraction of land/water flag
 *   30NOV2007  15892        B. Le        1.5.x.1  ECR A-103, EDRPR 1.8 CP 3
 *                                        updates -Format changes for CDFCB-X                 
 *                                        compliance - Remove references
 *                                        to granuleLevelFlagByte0.
 *   14Dec2007   16294       M. Lowe      Corrected to set out-of-range QF
 *   12JUN2008   17888       B. Bohannan  logger api refactoring
 *   12AUG2008   16724       W. Dare      EMD.2008.510.0006 Cirrus test update
 *   25NOV2008   19062       W. Dare      Corrected EVI missing input data qual 
 *                                        flags
 *   04DEC2008   18022       J. DeLotelle Modified to use Dictionary to 
 *                                        initialize output structures.
 *   16JUN2009   20338       S. Antolakis Removed the tempNdviExclBuf and
 *                                        tempEviExclBuf.
 *   19FEB2010   22732       B. Kowal     Updates For PCR 022732. 
 *                                        tempEviExclBuf.
 *   22FEB2010   22768       B. Kowal     Updates For PCR 22768. 
 *   03JUN2010   22732       E. Johnson   Changed methodology for assigning
 *                                        the non-availability QFs.
 *   22JUN2010   23667       E. Johnson   Added check for confidently clear 
 *                                        conditions before setting either the
 *                                        NDVI or EVI quality flags to 'high'
 *   21JAN2011               D. Doumakes  Modified for ADL: members of 
 *                                        VVI_DATA_TYPE were renamed
 *   30AUG2011   27763       W. Dare      Corrected I1 not available qf
 *   02MAY2013   34425       W. Dare      Correct SZA > 85 deg qf bounds check
 *   02FEB2014   37359       W. Dare      474-CCR-13-1218, QF update
 *
 * NOTES (MISCELLANEOUS) SECTION:
 * none
 *
 * --------- -------- -----------------------------------
 * <header review information>
 *
 **************************************************************************/

// Primary include file: this is where structures and constants are defined
#include <VegIndex.h>
#include <ProCmnViirsTrimTable.h>
#include <ProCmnScaler.h>
#include <ProCmnAlgorithm.h>
#include <ProEdrViirsSurfReflectGbl.h>
#include <ProCmnLogger.h>
#include <FloatCompare.h>
#include <AutoGeneratedViirsProductsGbl.h>
#include <ProEdrViirsVIStruct.h>

Int32 Calculate_VegIndex( 
    VVI_DATA_TYPE* viData, 
    Int32* stopPtr 
)
{
    DEBUG_MED((ProCmnLogger::getLogger()), "Calculate_VegIndex Called");

    Float32 ndvi       = NA_FLOAT32_FILL;
    Float32 evi        = NA_FLOAT32_FILL;
    Float32 tocndvi        = NA_FLOAT32_FILL;
    UInt8     night      = 0;
    UInt8     landWtr      = 0;
    UInt8     cloudy     = 0;
    UInt8     thinCirrus = 0;
    UInt8     glint      = 0;
    UInt8     heavyAot   = 0;
    UInt8     szaStrat   = 0;
    UInt8     szaExcl    = 0;
    UInt8     szaLow     = 0;

    /* LPEATE additions */
    UInt8    snowIce     = 0; // LPEATE added 06/07/2013
    UInt8    adjClouds   = 0; // LPEATE added 06/07/2013
    UInt8    aeroQuantity= 0; // LPEATE added 06/07/2013
    UInt8    cldShadow   = 0; // LPEATE added 06/07/2013
    
    Float32 denom        = 0.0;
    bool    badInputFlag;
    bool    eviOutOfRange;

    Int32 startCol = 0;
    Int32 endCol = 0;
    Int32 imgRow = 0;
    Int32 imgCol = 0;
    ProCmnViirsTrimTable *trimTable = ProCmnViirsTrimTable::getInstance();
    
    // determine max number of rows in current granule
    const Int32 rowMax = viData->geo->act_scans * M_DETECTORS;

    for( Int32 modRow = 0; modRow < rowMax; modRow++ )
    {
        // check for the stop call back
        if( *stopPtr == PROCESS_STOP )
        {
            return (PRO_STOP);
        }

        startCol = trimTable->getModLowerBound(modRow); 
        endCol = trimTable->getModUpperBound(modRow);

        for( Int32 modCol = startCol; modCol <= endCol; modCol++ )
        {
            // Apply computations to all non-ocean daytime pixels that are not 
            // cloudy or are obscured only by thin cirrus and/or aerosols
            night      = (viData->surfReflectIP->qfbyte0[modRow][modCol] & 
                    NIGHT_MASK) >> SR_DAY_NIGHT_SHIFT;
            landWtr    = viData->surfReflectIP->qfbyte1[modRow][modCol] & 
                VI_LAND_WATER_MASK;
            cloudy     = (viData->surfReflectIP->qfbyte0[modRow][modCol] & 
                    CLOUDY_MASK) >> SR_CLOUDY_SHIFT;
            thinCirrus = (viData->surfReflectIP->qfbyte1[modRow][modCol] & 
                    THIN_CIRRUS_REFLECT_MASK) >> SR_THIN_CIR_REFLECT_SHIFT;
            glint      = (viData->surfReflectIP->qfbyte0[modRow][modCol] &
                SUN_GLINT_MASK) >> SR_GLINT_SHIFT;;
            heavyAot   = (viData->surfReflectIP->qfbyte1[modRow][modCol] &
                    HEAVY_AEROSOL_MASK) >> SR_HEAVY_AERO_SHIFT;

            // LPEATE additions
            // New, additional QF bits being copied from SR QFs
            snowIce     = (viData->surfReflectIP->qfbyte6[modRow][modCol] &
                SR_SNOW_ICE_MASK) >> SR_SNOW_ICE_SHIFT;
			adjClouds   = (viData->surfReflectIP->qfbyte6[modRow][modCol] &
                SR_ADJ_CLOUDS_MASK) >> SR_ADJ_CLOUDS_SHIFT;
            aeroQuantity= (viData->surfReflectIP->qfbyte6[modRow][modCol] &
                SR_AERO_QUANTITY_MASK) >> SR_AERO_QUANTITY_SHIFT;
			cldShadow  = (viData->surfReflectIP->qfbyte1[modRow][modCol] &
                SHADOW_MASK) >> SR_SHADOW_SHIFT;
            // End of LPEATE addition

            // Loop over imagery pixels within moderate pixel 
            for( imgRow = modRow*2; imgRow <= modRow*2+1; imgRow++ )
            {
                for( imgCol = modCol*2; imgCol <= modCol*2+1; imgCol++ )
                {
                    // initialize temp vars
                    szaLow = 0;
                    szaExcl = 0;
                    szaStrat = 0;
                    badInputFlag = false;
                    eviOutOfRange = false;
                    
                    // Cloud Confidence - copied from SR IP
                    (viData->viirsVegIndexIP->qfbyte1[imgRow][imgCol]) |=
                        (cloudy << VI_CLOUD_SHIFT);

                    // Thin Cirrus - copied from SR IP
                    viData->viirsVegIndexIP->qfbyte1[imgRow][imgCol] |= 
                        (thinCirrus << VI_THIN_CIR_REFL_SHIFT);

                    // Land/Water Backgroud - copied from SR IP
                    viData->viirsVegIndexIP->qfbyte1[imgRow][imgCol] |=
                        landWtr;

                    // Sun Glint - copied from SR IP
                    viData->viirsVegIndexIP->qfbyte1[imgRow][imgCol] |= 
                        (glint << VI_SUN_GLINT_SHIFT);

                    // Excl - AOT > 1.0 - copied from SR IP
                    viData->viirsVegIndexIP->qfbyte2[imgRow][imgCol] |=
                        (heavyAot << VI_AOT_EXCL_SHIFT);

                    // SZA < 65 Deg
                    if( viData->geo->solar_zenith_angle[imgRow][imgCol] < 
                            viData->coeffs->SZA_LOW )
                    {
                        szaLow = 1;
                    }

                    // Excl - SZA > 85 Deg
                    if( viData->geo->solar_zenith_angle[imgRow][imgCol] > 
                        viData->coeffs->SZA_HI )
                    {
                        viData->viirsVegIndexIP->qfbyte2[imgRow][imgCol] 
                           |= VI_SZA_EXCL_MASK;

                        // SZA range exceeded    
                        szaExcl = 1;
                    }

                    // Stratification - 65 Deg >= SZA <= 85 Deg
                    if( viData->geo->solar_zenith_angle[imgRow][imgCol] >= 
                            viData->coeffs->SZA_LOW &&
                        viData->geo->solar_zenith_angle[imgRow][imgCol] <= 
                            viData->coeffs->SZA_HI )
                    {
                        viData->viirsVegIndexIP->qfbyte2[imgRow][imgCol] 
                           |=  VI_SZA_STRAT_MASK;
                    }

                    // LPEATE additions
                    // Snow/Ice - copied from SR IP
                    (viData->viirsVegIndexIP->qfbyte2[imgRow][imgCol]) |=
                      (snowIce << VI_SNOW_ICE_SHIFT);

                    // Adjacent to Clouds - copied from SR IP
                    (viData->viirsVegIndexIP->qfbyte2[imgRow][imgCol]) |=
                      (adjClouds << VI_ADJ_CLOUDS_SHIFT);

                    // Aerosol Quantity - copied from SR IP
                    (viData->viirsVegIndexIP->qfbyte2[imgRow][imgCol]) |=
                      (aeroQuantity << VI_AERO_QUANTITY_SHIFT);

                    // Cloud Shadow - copied from SR IP
                    (viData->viirsVegIndexIP->qfbyte2[imgRow][imgCol]) |=
                      (cldShadow << VI_CLD_SHADOW_SHIFT);

                    // End of LPEATE addition


                    // Night scans need to be filled with data does not exist
                    // fill value
                    if(viData->geo->scan_mode[(imgRow/I_DETECTORS)] == NIGHT)
                    {
                        viData->viirsVegIndexIP->TOA_NDVI[imgRow][imgCol] = 
                            VDNE_FLOAT32_FILL;
                        viData->viirsVegIndexIP->TOC_EVI[imgRow][imgCol] = 
                            VDNE_FLOAT32_FILL;
                        continue;
                    }

                    // check both channels of surface reflectance for fill 
                    // value skip evi calculation if fill detected 
                    if ( viData->surfReflectIP->i1[imgRow][imgCol] <= 
                            FLOAT32_FILL_TEST )
                    {
                        // set qual flag
                        viData->viirsVegIndexIP->
                            qfbyte0[imgRow][imgCol] |=
                            VI_I1_SURF_REFL_AVAIL_MASK;
                        badInputFlag = true;
                    }

                    if ( viData->surfReflectIP->i2[imgRow][imgCol] <= 
                            FLOAT32_FILL_TEST )
                    {
                        // set qual flag
                        viData->viirsVegIndexIP->
                            qfbyte0[imgRow][imgCol] |= 
                            VI_I2_SURF_REFL_AVAIL_MASK;

                        badInputFlag = true;
                    }

                    if ( viData->surfReflectIP->m3[modRow][modCol] <=
                            FLOAT32_FILL_TEST )
                    {
                        // set qual flag
                        viData->viirsVegIndexIP->
                            qfbyte0[imgRow][imgCol] |=
                            VI_M3_SURF_REFL_AVAIL_MASK;   

                        badInputFlag = true;
                    }

                    // process only applicable pixels
                    if( (landWtr != VI_OCEAN_MASK) && !night) 
                    {    
                        // initialize ndvi and evi to fill values
                        ndvi = NA_FLOAT32_FILL;
                        evi  = NA_FLOAT32_FILL;
						tocndvi = NA_FLOAT32_FILL;

                        // Check for bad inputs from previous examination of 
                        //  the i1, i2, and m3 bands from surface reflectance
                        if (!badInputFlag)
                        {
                            denom = viData->surfReflectIP->i2[imgRow][imgCol]   
                                + ( viData->coeffs->EVI_I1 * 
                                    viData->surfReflectIP->i1[imgRow][imgCol] ) 
                                - ( viData->coeffs->EVI_M3 * 
                                    viData->surfReflectIP->m3[modRow][modCol] )
                                +   viData->coeffs->EVI_C;

                            if (!Float32Compare::equal(denom, 0.0))
                            {
                                // Top of canopy EVI calculation
                                evi = ( 1.0 + viData->coeffs->EVI_C ) *
                                    ( viData->surfReflectIP->i2[imgRow][imgCol]
                                   -  viData->surfReflectIP->i1[imgRow][imgCol])
                                   / denom;

                                // The following lines are added to bring the 
                                // code and the specs in agreement. The specs 
                                // states that the range of EVI should be 
                                // [-1;4]. So any pixel beyond that range is 
                                // flagged by being set to the fill value 
                                // SOUB_FLOAT32_FILL.
                                if( (evi > viData->coeffs->EVI_MAX) || 
                                    (evi < viData->coeffs->EVI_MIN) )
                                {
                                    // evi overall qual flag already 
                                    // initialized to low, no
                                    // need to set it again
                                    evi = SOUB_FLOAT32_FILL;

                                    // set EVI out of range flag to true
                                    eviOutOfRange = true;

                                    // set EVI QF on (1) to indicate out of 
                                    // range
                                    viData->viirsVegIndexIP->
                                           qfbyte0[imgRow][imgCol] |= 
                                             VI_EVI_RANGE_MASK;
                                }
                                else
                                {
                                    // set EVI quality to high(1) if ALL these 
                                    // conditions are met:
                                    // NOTE: conditions 1-4, and 7 must be true
                                    //  to hit this code
                                    //  1) I1 Surface reflectance flag = avail 
                                    //  2) I2 Surface reflectance flag = avail 
                                    //  3) M3 Surface reflectance flag = avail 
                                    //  4) Cloud Confidence flag = confidently 
                                    //     clear 
                                    //  5) Thin Cirrus flag = no thin cirrus 
                                    //  6) Solar Zenith Angle < 65 deg 
                                    //  7) Sun glint (Geometry based) = none  
                                    //  8) EVI range flag = in range 
                                    if( !thinCirrus && !glint && szaLow &&
                                        (cloudy == VI_CLOUD_MASK_CONF_CLEAR))
                                    {
                                        // set overall evi quality flag to high
                                        viData->viirsVegIndexIP->
                                           qfbyte0[imgRow][imgCol] |= 
                                           VI_EVI_QUAL_MASK;
										// set overall ndvi quality flag to high
                                    }
                                    eviOutOfRange = false;
                                }
                            }
                            // problem with denominator
                            else
                            {
                                evi = NA_FLOAT32_FILL;
							} 
                        }
                        // channel incorrect
                        else
                        {
                            evi = NA_FLOAT32_FILL;
						}

                        // check both channels of sdr reflectance for fill value
                        // skip ndvi calculation if fill detected 
                        if( (viData->reflI1->Bt_refl[imgRow][imgCol] > 
                             FLOAT32_FILL_TEST) && 
                            (viData->reflI2->Bt_refl[imgRow][imgCol] > 
                             FLOAT32_FILL_TEST) )
                        {
                            // check for 0.0 value
                            denom = viData->reflI2->Bt_refl[imgRow][imgCol] + 
                                    viData->reflI1->Bt_refl[imgRow][imgCol];

                            // set NDVI quality to high(1) if ALL these 
                            // conditions are met:
                            // NOTE: conditions 1-4 must be true to hit here
                            //  1) I1 TOA reflectance flag = available
                            //  2) I2 TOA reflectance flag = available 
                            //  3) Cloud Confidence flag = confidently clear &
                            //  4) Thin Cirrus flag = no thin cirrus & 
                            //  5) Solar Zenith Angle < 65 deg & 
                            //  6) Sun glint (Geometry based) = none  
                            if((cloudy == VI_CLOUD_MASK_CONF_CLEAR) && 
                               !thinCirrus && szaLow && !glint )
                            {
                                // set overall ndvi quality flag to high
                                viData->viirsVegIndexIP->
                                   qfbyte0[imgRow][imgCol] |= 
                                   VI_NDVI_QUAL_MASK;    
                            }

                            if (!Float32Compare::equal(denom, 0.0)) 
                            {
                                // Top of atmosphere NDVI calculation
                                ndvi = ( viData->reflI2->Bt_refl[imgRow][imgCol]
                                      - viData->reflI1->Bt_refl[imgRow][imgCol])
                                      / denom;

                                // The following lines are added to bring the 
                                // code and the specs in agreement. The specs 
                                // states that the range of NDVI should be 
                                // [-1;1]. So any pixel beyond that range is 
                                // flagged by a SOUB_FLOAT32_FILL value
                                if( ( ndvi > viData->coeffs->NDVI_MAX ) || 
                                    ( ndvi < viData->coeffs->NDVI_MIN ) )
                                {
                                    // ndvi overall qual flag already 
                                    // initialized to low, no
                                    // need to set it again
                                    ndvi = SOUB_FLOAT32_FILL;
                                }
                            }
                        }
                        else
                        {
                            // check if reflectance I1 is bad
                            if(viData->reflI1->Bt_refl[imgRow][imgCol] < 
                               FLOAT32_FILL_TEST)
                            {
                                viData->viirsVegIndexIP->
                                   qfbyte0[imgRow][imgCol] |=
                                   VI_I1_TOA_REFL_AVAIL_MASK;
                            }
                            // check if reflectance I2 is bad
                            if(viData->reflI2->Bt_refl[imgRow][imgCol] < 
                               FLOAT32_FILL_TEST)
                            {
                                viData->viirsVegIndexIP->
                                   qfbyte0[imgRow][imgCol] |=
                                   VI_I2_TOA_REFL_AVAIL_MASK;    
                            }
                            // fill NDVI regardless of which band is missing
                            ndvi = NA_FLOAT32_FILL;
                        }
						
						// check both channels of sdr reflectance for fill value
                        // skip ndvi calculation if fill detected 
                        if( (viData->surfReflectIP->i1[imgRow][imgCol] > 
                            FLOAT32_FILL_TEST)  && 
                            (viData->surfReflectIP->i2[imgRow][imgCol] > 
                            FLOAT32_FILL_TEST) )
                        {
                            // check for 0.0 value
                            denom = ( viData->surfReflectIP->i2[imgRow][imgCol]
									+  viData->surfReflectIP->i1[imgRow][imgCol]);

                            // set TOC NDVI quality to high(1) if ALL these 
                            // conditions are met:
                            // NOTE: conditions 1-4 must be true to hit here
                            //  1) I1 surface reflectance flag = available
                            //  2) I2 surface reflectance flag = available 
                            //  3) Cloud Confidence flag = confidently clear &
                            //  4) Thin Cirrus flag = no thin cirrus & 
                            //  5) Solar Zenith Angle < 65 deg & 
                            //  6) Sun glint (Geometry based) = none  
                            if((cloudy == VI_CLOUD_MASK_CONF_CLEAR) && 
                               !thinCirrus && szaLow && !glint )
                            {
                                // set overall ndvi quality flag to high
                                viData->viirsVegIndexIP->
                                   qfbyte3[imgRow][imgCol] |= 
                                   VI_TOCNDVI_QUAL_MASK;    
                            }

                            if (!Float32Compare::equal(denom, 0.0)) 
                            {
                                // Top of Canopy NDVI calculation
                                tocndvi = ( viData->surfReflectIP->i2[imgRow][imgCol]
									-  viData->surfReflectIP->i1[imgRow][imgCol])
									/ denom;

                                // The following lines are added to bring the 
                                // code and the specs in agreement. The specs 
                                // states that the range of NDVI should be 
                                // [-1;1]. So any pixel beyond that range is 
                                // flagged by a SOUB_FLOAT32_FILL value
                                if( ( tocndvi > viData->coeffs->NDVI_MAX ) || 
                                    ( tocndvi < viData->coeffs->NDVI_MIN ) )
                                {
                                    // ndvi overall qual flag already 
                                    // initialized to low, no
                                    // need to set it again
                                    tocndvi = SOUB_FLOAT32_FILL;
                                }
                            }
                        }
                        else
                        {
                            // fill TOC NDVI regardless of which band is missing
                            tocndvi = NA_FLOAT32_FILL;
                        }

                        // Store ndvi and evi
                        viData->viirsVegIndexIP->TOA_NDVI[imgRow][imgCol] = 
                            ndvi;
                        viData->viirsVegIndexIP->TOC_EVI[imgRow][imgCol] = 
                            evi;
						viData->viirsVegIndexIP->TOC_NDVI[imgRow][imgCol] = 
                            tocndvi;
                    }
                    else
                    {
                        // set the NDVI and EVI to fill values using the same 
                        // computations
                        // as the valid pixels
                        viData->viirsVegIndexIP->TOA_NDVI[imgRow][imgCol] = 
                            NA_FLOAT32_FILL;
                        viData->viirsVegIndexIP->TOC_EVI[imgRow][imgCol] = 
                            NA_FLOAT32_FILL;
						viData->viirsVegIndexIP->TOC_NDVI[imgRow][imgCol] = 
                            NA_FLOAT32_FILL;
                    }

                    // Overide all values when geolocation
                    // is fill type ELLIPSOID_FLOAT32_FILL
                    if(Float32Compare::equal(viData->geo->longitude[imgRow][imgCol],
                                            ELLIPSOID_FLOAT32_FILL))
                    {
                        viData->viirsVegIndexIP->TOA_NDVI[imgRow][imgCol] =
                                ELLIPSOID_FLOAT32_FILL;
                        viData->viirsVegIndexIP->TOC_EVI[imgRow][imgCol] =
                                ELLIPSOID_FLOAT32_FILL;
						viData->viirsVegIndexIP->TOC_NDVI[imgRow][imgCol] =
                                ELLIPSOID_FLOAT32_FILL;
                    }

                } // end img col
            } // end img row
        } // end mod col
    } // end mod row

    return( PRO_SUCCESS );
}
