/* BEGIN COPYRIGHT */
/* END COPYRIGHT */
/* BEGIN GOVERNMENT_RIGHTS */
/***********************************************************************
 * UNLIMITED RIGHTS
 * This material may be reproduced by or on behalf of the
 * U.S. Government pursuant to the copyright license under the DFAR
 * 252.227-7014 which was made part of the above contract.
 ***********************************************************************/
/* END GOVERNMENT_RIGHTS */
/* BEGIN ITAR */
/* END ITAR */

/* BEGIN CATEGORY */
/************************************************************************** 
 * CATEGORY: 1 (see SEN for definitions)
 *
 * (ProEdrViirsVI.cpp)
 *
 * Refer to Software Standards and Practices Manual (SSPM) for 
 * coding standards.
 *
 **************************************************************************/
/* END CATEGORY */

/**************************************************************************
 * 
 * NAME: ProEdrViirsVI
 *
 * DESCRIPTION: The ProEdrViirsVI class is the subclass of the ProCmnAlgorithm 
 *              class for the viirs vegetation index algorithm, which contains 
 *              the overridden methods setupDataItems and doProcessing.
 *
 *
 * USAGE: See header file for more information.
 *
 * Parameter         Type          I/O    Description
 * -----------       -----         ---    ----------------
 * See the method descriptions located in the .h file for details.
 *
 *
 * REFERENCES:
 * Y2400  - Vegetation Index Algorithm Theoretical Basis Document
 * Y2474  - Land Module Level Software Architecture Document
 * Y2499  - VIIRS Vegetation Index Unit Level Detailed Design Document
 * Y10883 - VIIRS Land Module Data Dictionary
 * D36951 - Operational Algorithm Document for the VIIRS Vegetation Index EDR 
 *          Software

 *
 * LIMITATIONS: 
 * none
 *
 *
 * HISTORY:
 *   DATE:       PR#         AUTHOR      Description
 *   --------    ------      --------    -----------------
 *   19SEP2003               mhoover     Initial Version
 *   25FEB2004   1306        mhoover     Code changes to IPO model
 *   31MAR2004   1778        mhoover     Added comments, changed 
 *   28APR2004   2051        mhoover     Changed 
 *                                       INF_STATUSTYPE_EDR_FAIL to
 *                                       INF_STATUSTYPE_FREE_FORM*
 *   16FEB2005               C. Erkes    Updated input/output from vector
 *                                       to map, updated shortNames,
 *                                       updated coeffs structure to use
 *                                       ING structure, updated prolog
 *   06APR2005               M. Hoover   Removed setVersion and removeOutputItem
 *   15MAY2005               C. Erkes    Updated ProCmnDynamicDataItemt to 
 *                                       ProCmnAuxDataItem
 *   26MAY2005               B. Nye      Updated ITAR markings
 *   21JUN2005               B. Nye      Modified doProcessing to use
 *                                       assignDataPtr instead of static_cast
 *   20DEC2005               S. Stewart  Added shell granule
 *                                       functionality 
 *   17MAR2006               D. Stuhmer  Updated INF status and
 *                                       details messages.
 *   28MAR2006   9351        D. Hrivnak  Updated debug log levels
 *   07JUL2006   011677      T. Hahn     Replaced getShortName() with
 *                                         getInputShortName() or
 *                                         getOutputShortName()
 *   18JUL2006   11135       M.Schmidt   Add fill DQN to populate metadata
 *   31JUL2006               C. Erkes    Updated setupDataItems() &
 *                                       doProcessing() to take in
 *                                       Geolocation as an input
 *   16AUG2006               C. Erkes    Updated DQTT code to use
 *                                       ProCmnPercentQualityFlag()
 *   16MAR2007               B. Henderson Updates for DM_BADADDRESS
 *   22MAR2007               B. Henderson Updated for ProCmnConvertor removal
 *                                       from ProCmnSdrItem constructor
 *   06MAY2007               Henderson   Updated to use ProCmnProductDictionary
 *                                       for scaling and fill tests
 *   03APR2008   17325       B. Bohannan  1.5X.1   magic number removal
 *   06MAY2008   17241       W. Dare     Moved shellInputAccept to
 *                                       ProCmnProductDictionary
 *   25MAY2008   17333       B. Bohannan 1.5x1   ProCmnDataItem refactor
 *   12JUN2008   17888       B. Bohannan logger api refactoring
 *   20JUN2008  017911       B. Bohannan pro cmn algo refactor stage 1
 *   08JUL2008   17911       B. Bohannan pro cmn algo refactor stage 2
 *   22JUL2008  017911       B. Bohannan algorithm refactor step 3
 *   28JUL2008               J. Gibbs    Modifications for the DQN/DQTT
 *                                       rework.
 *   12AUG2008               T. Bosley   Fixed a few bugs, cleaned up the code, 
 *                                       added comments where needed for DQN
 *   04DEC2008   18898       J. Gibbs    Modified the addToQualityMap
 *                                       signature to pass the default
 *                                       value
 *   16JUN2009   20338       S. Antolakis Removed the tempNdviExclBuf and
 *                                       tempEviExclBuf and tempImgSize.  
 *   09DEC2009   21642       J. Gibbs    Added Ndvi and Evi exclusion summary
 *   05MAR2010   22883       B. Kowal    Implemented TM NP-EMD-2010.510.0005
 *   31MAR2010   23012       B. Kowal    Updated the method used to determine
 *                                       the value of the granule-level
 *                                       no land in granule quality flag.
 *   07APR2010   23012       B. Kowal    Added a function that will examine
 *                                       the land/water quality flag for
 *                                       every non-trim pixel before
 *                                       determining how to set the
 *                                       No-Land-In-Granule Summary Quality
 *                                       Flag.
 *   24JUN2010   23012       E. Johnson  Modified the above to account for short
 *                                       granules.
 *   21JAN2010               D. Doumakes Modified for ADL, and file renamed
 *
 *
 * NOTES (MISCELLANEOUS) SECTION:
 * none
 *
 * --------- -------- -----------------------------------
 * <header review information>
 *
 **************************************************************************/

#include <ProEdrViirsVIStruct.h>
#include <ProEdrViirsVI.h>
#include <AutoGeneratedProEdrViirsVI.h>
#include <ProCmnSDRItem.h>
#include <VegIndex.h>
#include <ProEdrViirsVIGbl.h>
#include <ProCmnLogger.h>
#include <ProCmnCalculateSummary.h>
#include <ProCmnViirsTrimTable.h>
#include <ProCmnEDROutputItem.h>

//-----------------------------------------------------------------------------
// Constructor
//-----------------------------------------------------------------------------
ProEdrViirsVI::ProEdrViirsVI() : 
    myCallerID_("ProEdrViirsVI")
{
   viDataType_.geo             = 0;
   viDataType_.reflI1          = 0;
   viDataType_.reflI2          = 0;
   viDataType_.surfReflectIP   = 0;
   viDataType_.coeffs          = 0;
   viDataType_.viirsVegIndexIP = 0;
}

//-----------------------------------------------------------------------------
// abstract superclass implementation
//-----------------------------------------------------------------------------
/* virtual */ const std::string& ProEdrViirsVI::getRoutine() const
{
   return myCallerID_.getClassName();
}

//-----------------------------------------------------------------------------
// Destructor
//-----------------------------------------------------------------------------
/* virtual */ProEdrViirsVI::~ProEdrViirsVI()
{
}

//-----------------------------------------------------------------------------
// The processing of the input data and generation of the output data
// by the viirs vegetation index algorithm is done in this method. This calls 
// the main function generate_vegindex to do the actual processing.
//-----------------------------------------------------------------------------
/* virtual */ Int32 ProEdrViirsVI::doProcessing()
{
   ProCmnMethodAudit methodAudit(
         this,
         "ProEdrViirsVI",
         "doProcessing",
         getMethodContext());

   Int32 status = PRO_SUCCESS;

   // Get imagery geolocation from DMS
   viDataType_.geo = inputGeolocation_Img_TC_.start_Ptr;

   // Get reflectance for imagery band 1 from DMS 
   viDataType_.reflI1 = inputImage_Band01_.start_Ptr;

   // Get reflectance for imagery band 2 from DMS 
   viDataType_.reflI2 = inputImage_Band02_.start_Ptr;

   // Get surface reflectance from DMS 
   viDataType_.surfReflectIP = inputSurfReflect_IP_.start_Ptr;

   // Get Processing coefficients from DMS
   viDataType_.coeffs = inputVI_Pro_Coeffs_.start_Ptr;

   //VI output
   viDataType_.viirsVegIndexIP = outputVegIndex_EDR_.start_Ptr;

   // call science algorithm    
   status = Calculate_VegIndex( &viDataType_, &stopStatus_ );

   if( status == PRO_STOP )
   {
      methodAudit.setRootStatus(status, "INF stop request",
            __FILE__, __LINE__);
      infStatusGeneral(INF_STATUSTYPE_PRO_FREE_FORM,
            "Stop call back received from INF in VIIRS VegIndex algorithm");
   }

   else if( status == PRO_FAIL )
   {
      methodAudit.setRootStatus(status, "Calculate_VegIndex() call",
            __FILE__, __LINE__);
      infStatusGeneral(INF_STATUSTYPE_PRO_FREE_FORM,
            "Errors with VIIRS VegIndex Algorithm");
   }

   return (status);
}

//--------------------------------------------------------------------------
// my caller identity
//--------------------------------------------------------------------------
const ProCmnCallerID *ProEdrViirsVI::callerID(int linenum) const
{
   // abstract meaning of const
   ((ProEdrViirsVI*)this)->myCallerID_.setLineNumber(linenum);

   return &myCallerID_;
}

/* virtual */ Int32 ProEdrViirsVI::setupDataItems()
{
   ProCmnMethodAudit methodAudit(
       this,
       "ProEdrViirsVI",
       "setupDatItems",
       getMethodContext() );

   Int32 status = ProCmnAlgorithm::setupDataItems();
   if ( PRO_SUCCESS == status )
   {
       // Add the quality metadata names and default values to the map.
       // This is needed for creating shells at night.
       ProCmnEDROutputItem * vegIndexEdrPtr = 
           static_cast<ProCmnEDROutputItem*>(getOutputItem(VEGINDEX_EDR));
       vegIndexEdrPtr->addToQualityMap(VI_EVI_QUAL,DEFAULT_QUAL_MAX_VAL);
       vegIndexEdrPtr->addToQualityMap(VI_NDVI_QUAL,DEFAULT_QUAL_MAX_VAL);
	   vegIndexEdrPtr->addToQualityMap(VI_TOCNDVI_QUAL,DEFAULT_QUAL_MAX_VAL);
       vegIndexEdrPtr->addToQualityMap(VI_EVI_EXCL_SUM);
       vegIndexEdrPtr->addToQualityMap(VI_NDVI_EXCL_SUM);
	   vegIndexEdrPtr->addToQualityMap(VI_TOCNDVI_EXCL_SUM);
       vegIndexEdrPtr->addToQualityMap(VI_EXCL_NO_LAND);
   }

   return status;
}
